@@ -3,7 +3,7 @@
  Documentation generator for the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: June 19, 2011
+ Last Change: June 23, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -29,6 +29,7 @@ local SOURCES = [[
   http.c
   proc.c
   shm.c
+  signal.c
   str.c
   thread.c
   thread_queue.c
@@ -241,16 +242,11 @@ end
 -- Convert documentation comments to Markdown hypertext. {{{1
 
 local function findrelease()
-  local handle = assert(io.open 'src/apr.lua')
-  local version, release
-  for line in handle:lines() do
-    version = version or line:match "^apr%._VERSION = '(.-)'"
-    release = release or line:match "^apr%._RELEASE = '(.-)'"
+  for line in io.lines 'src/apr.lua' do
+    local version = line:match "^apr%._VERSION = '(.-)'"
+    if version then return version end
   end
-  assert(handle:close())
-  assert(version, "Failed to determine Lua/APR version number!")
-  assert(release, "Failed to determine Lua/APR release number!")
-  return version .. '-' .. release
+  assert(false, "Failed to determine Lua/APR version number!")
 end
 
 local blocks = { trim([[
@@ -395,6 +391,7 @@ local bsignore = {
   ['apr.parse_query_string'] = true,
   ['apr.platform_get'] = true,
   ['apr.proc_fork'] = true,
+  ['apr.signal_names'] = true,
   ['apr.strfsize'] = true,
   ['apr.type'] = true,
   ['apr.uri_port_of_scheme'] = true,
