@@ -1,7 +1,7 @@
 /* Multi threading module for the Lua/APR binding.
  *
  * Author: Peter Odding <peter@peterodding.com>
- * Last Change: May 15, 2011
+ * Last Change: June 30, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
  *
@@ -24,12 +24,10 @@
  *    protect object access with a thread safe lock. This will probably be
  *    fixed in the near future (hey, I said it was experimental)
  *
- *  - When a thread object is garbage collected while the thread is still
- *    running, the Lua/APR binding will join the thread using an internal mutex
- *    and condition variable from inside the garbage collection handler. This
- *    is to avoid that the parent Lua state and the operating system process
- *    exit while the child is still running, because this can lead to hard to
- *    debug segmentation faults deep inside [libc] [libc]
+ *  - When you start a thread and let it get garbage collected without having
+ *    called `thread:join()`, the thread will be joined for you (because
+ *    failing to do so while the main thread is terminating can crash the
+ *    process)
  *
  * [Lua_state]: http://www.lua.org/manual/5.1/manual.html#lua_State
  * [threading]: http://en.wikipedia.org/wiki/Thread_%28computer_science%29
@@ -40,29 +38,16 @@
 #include <lualib.h>
 #include <apr_strings.h>
 #include <apr_portable.h>
-#include <apr_thread_cond.h>
 
 #if APR_HAS_THREADS
 
-/* I've left the debugging messages in for now in case the mutex and condition
- * variable handling turns out to be buggy after publishing this code (you'll
- * have to define LUA_APR_DEBUG_THREADS=1 before anything is printed). */
-
-#ifndef LUA_APR_DEBUG_THREADS
-#define LUA_APR_DEBUG_THREADS 0
-#endif
-
-#define LUA_APR_THREAD_MSG(msg, ...) \
-  if (LUA_APR_DEBUG_THREADS) \
-    LUA_APR_DBG(msg, __VA_ARGS__)
-
-/* The thread which initialized the Lua/APR binding. */
-static apr_os_thread_t initial_thread;
-
 /* Private parts. {{{1 */
 
 const char *status_names[] = { "init", "running", "done", "error" };
 
+#define check_thread(L, idx) \
+  (lua_apr_thread*)check_object(L, idx, &lua_apr_thread_type)
+
 #define thread_busy(T) \
   ((T)->status == TS_INIT || (T)->status == TS_RUNNING)
 
@@ -75,20 +60,9 @@ typedef struct {
   apr_threadattr_t *attr;
   void *input, *output;
   volatile thread_status_t status;
-  int detached, joined;
-  apr_os_thread_t parent;
+  int joined;
 } lua_apr_thread;
 
-/* check_thread(L, idx, check_joinable) {{{2 */
-
-static lua_apr_thread *check_thread(lua_State *L, int idx, int check_joinable)
-{
-  lua_apr_thread *object = check_object(L, idx, &lua_apr_thread_type);
-  if (check_joinable && object->detached)
-    luaL_error(L, "attempt to join a detached thread");
-  return object;
-}
-
 /* error_handler(state) {{{2 */
 
 /* Copied from lua-5.1.4/src/lua.c where it's called traceback() */
@@ -115,15 +89,13 @@ static int error_handler(lua_State *L)
 
 /* thread_destroy(thread) {{{2 */
 
-static void thread_destroy(lua_State *L, lua_apr_thread *thread)
+static void thread_destroy(lua_apr_thread *thread)
 {
   if (object_collectable((lua_apr_refobj*)thread)) {
-    /* If the thread exited with an error and the user didn't check the
-     * result using thread:join() we print the error message to stderr. */
     free(thread->input);
     free(thread->output);
   }
-  release_object(L, (lua_apr_refobj*)thread);
+  release_object((lua_apr_refobj*)thread);
 }
 
 /* thread_runner(handle, thread) {{{2 */
@@ -141,6 +113,7 @@ static void* lua_apr_cc thread_runner(apr_thread_t *handle, lua_apr_thread *thre
   /* Start by creating a new Lua state. */
   if ((L = luaL_newstate()) == NULL) {
     status = TS_ERROR;
+    thread->output = strdup("Failed to create Lua state");
   } else {
     luaL_openlibs(L); /* load standard library */
     lua_settop(L, 0); /* normalize stack */
@@ -151,7 +124,7 @@ static void* lua_apr_cc thread_runner(apr_thread_t *handle, lua_apr_thread *thre
       thread->output = strdup(lua_tostring(L, -1));
       status = TS_ERROR;
     } else {
-      lua_replace(L, 2);
+      lua_replace(L, 2); /* replace string with function */
       thread->status = TS_RUNNING;
       if (lua_pcall(L, lua_gettop(L) - 2, LUA_MULTRET, 1)) {
         thread->output = strdup(lua_tostring(L, -1));
@@ -164,23 +137,8 @@ static void* lua_apr_cc thread_runner(apr_thread_t *handle, lua_apr_thread *thre
     lua_close(L);
   }
 
-  thread_destroy(NULL, thread);
-  threads_decrement(L);
-
-  /* If an error occurred, make sure the user sees it. */
-  if (status == TS_ERROR)
-    fprintf(stderr, "Lua/APR thread exited with error: %s\n", (char*)thread->output);
-
   thread->status = status;
-  /* XXX Theoretically, if the parent thread inspects thread->status at this
-   * point, the child thread hasn't terminated yet. If the parent thread were
-   * to terminate before the child thread, a crash would most likely result...
-   * I don't like this one bit but can't see how there's any around it!
-   *
-   * (I've debugged several crash bugs caused by race conditions between parent
-   * and child threads before publishing this code and wound up running the
-   * thread_queue.lua tests for a 1000 iterations before I decided to publish
-   * this code). */
+  thread_destroy(thread);
   apr_thread_exit(handle, APR_SUCCESS);
 
   /* To make the compiler happy. */
@@ -209,15 +167,11 @@ int lua_apr_thread_create(lua_State *L)
 
   /* Create the Lua/APR thread object. */
   thread = new_object(L, &lua_apr_thread_type);
-  if (thread == NULL)
-    goto fail;
-  thread = prepare_reference(L, &lua_apr_thread_type, (lua_apr_refobj*)thread);
   if (thread == NULL)
     goto fail;
   thread->status = TS_INIT;
-  thread->parent = apr_os_thread_current();
 
-  /* Create a memory pool for the thread structure. */
+  /* Create memory pool for thread (freed by apr_thread_exit()). */
   status = apr_pool_create(&thread->pool, NULL);
   if (status != APR_SUCCESS)
     goto fail;
@@ -235,15 +189,12 @@ int lua_apr_thread_create(lua_State *L)
   if (status != APR_SUCCESS)
     goto fail;
 
-  /* Remember that we started another thread. */
-  threads_increment(L);
-
   /* Return the thread object. */
   return 1;
 
 fail:
   if (thread != NULL)
-    thread_destroy(L, thread);
+    thread_destroy(thread);
   return push_error_status(L, status);
 }
 
@@ -259,25 +210,6 @@ int lua_apr_thread_yield(lua_State *L)
   return 0;
 }
 
-/* thread:detach() -> status {{{1
- *
- * Detach a thread to make the operating system automatically reclaim storage
- * for the thread once it terminates. On success true is returned, otherwise
- * nil followed by an error message is returned.
- */
-
-static int thread_detach(lua_State *L)
-{
-  lua_apr_thread *object;
-  apr_status_t status;
-
-  object = check_thread(L, 1, 0);
-  status = apr_thread_detach(object->handle);
-  if (status == APR_SUCCESS)
-    object->detached = 1;
-  return push_status(L, status);
-}
-
 /* thread:join() -> status [, result, ...] {{{1
  *
  * Block until a thread stops executing and return its result. If the thread
@@ -293,7 +225,7 @@ static int thread_join(lua_State *L)
   lua_apr_thread *object;
   apr_status_t status, unused;
 
-  object = check_thread(L, 1, 1);
+  object = check_thread(L, 1);
   lua_settop(L, 1);
 
   /* Don't join more than once. */
@@ -329,7 +261,7 @@ static int thread_status(lua_State *L)
 {
   lua_apr_thread *object;
 
-  object = check_thread(L, 1, 0);
+  object = check_thread(L, 1);
   lua_pushstring(L, status_names[object->status]);
 
   return 1;
@@ -339,7 +271,7 @@ static int thread_status(lua_State *L)
 
 static int thread_tostring(lua_State *L)
 {
-  lua_apr_thread *object = check_thread(L, 1, 0);
+  lua_apr_thread *object = check_thread(L, 1);
   lua_pushfstring(L, "%s (%s)",
       lua_apr_thread_type.friendlyname,
       status_names[object->status]);
@@ -350,106 +282,29 @@ static int thread_tostring(lua_State *L)
 
 static int thread_gc(lua_State *L)
 {
-  thread_destroy(L, check_thread(L, 1, 0));
-  return 0;
-}
+  lua_apr_thread *thread;
+  apr_status_t status, unused;
 
-/* Module initialization and finalization. {{{1 */
-
-static apr_pool_t *pool = NULL;
-static apr_thread_mutex_t *mutex;
-static apr_thread_cond_t *condition;
-static volatile int num_threads = 0;
-
-void threads_initialize(lua_State *L)
-{
-  apr_status_t status;
-
-  LUA_APR_THREAD_MSG("(L=%p) Initializing threading module!", L);
-
-  /* XXX Remember which thread initialized the Lua/APR binding, because this
-   * thread will have to "join" any child threads before it exits, otherwise
-   * nasty crashes can result. I say "join" because we actually use a mutex
-   * and condition variable instead of apr_thread_join() so that detached
-   * threads are also supported (though I'm not sure how useful this is). */
-  initial_thread = apr_os_thread_current();
-
-  status = apr_pool_create(&pool, NULL);
-  if (status == APR_SUCCESS)
-    apr_thread_mutex_create(&mutex, APR_THREAD_MUTEX_UNNESTED, pool);
-  if (status == APR_SUCCESS)
-    apr_thread_cond_create(&condition, pool);
-  if (status != APR_SUCCESS)
-    raise_error_status(L, status);
-}
-
-int threads_error(lua_State *L, apr_status_t status)
-{
-  if (status == APR_SUCCESS)
-    return 0;
-  if (L != NULL) {
-    raise_error_status(L, status);
-  } else {
-    char message[LUA_APR_MSGSIZE];
-    apr_strerror(status, message, count(message));
-    fprintf(stderr, "Lua/APR threading module internal error: %s\n", message);
-  }
-  return 1;
-}
-
-void threads_lock(lua_State *L)
-{
-  LUA_APR_THREAD_MSG("(L=%p) Enabling threading module lock!", L);
-  threads_error(L, apr_thread_mutex_lock(mutex));
-}
-
-void threads_unlock(lua_State *L, apr_status_t status)
-{
-  LUA_APR_THREAD_MSG("(L=%p) Disabling threading module lock!", L);
-  if (status == APR_SUCCESS)
-    status = apr_thread_mutex_unlock(mutex);
-  else
-    apr_thread_mutex_unlock(mutex);
-  if (L == NULL)
-      apr_pool_destroy(pool);
-  threads_error(L, status);
-}
-
-void threads_increment(lua_State *L)
-{
-  threads_lock(L);
-  LUA_APR_THREAD_MSG("(L=%p) Raising child thread count from %i to %i", L, num_threads, num_threads + 1);
-  num_threads++;
-  threads_unlock(L, APR_SUCCESS);
-}
-
-void threads_decrement(lua_State *L)
-{
-  threads_lock(L);
-  LUA_APR_THREAD_MSG("(L=%p) Lowering child thread count from %i to %i", L, num_threads, num_threads - 1);
-  num_threads--;
-  LUA_APR_THREAD_MSG("(L=%p) Signaling child thread termination", L);
-  threads_unlock(L, apr_thread_cond_signal(condition));
-}
-
-void threads_terminate()
-{
-  if (apr_os_thread_equal(initial_thread, apr_os_thread_current())) {
-    threads_lock(NULL);
-    LUA_APR_THREAD_MSG("%s", "Terminating threading module!");
-    while (num_threads > 0) {
-      LUA_APR_THREAD_MSG("%s", "Waiting for child thread termination signal..");
-      threads_error(NULL, apr_thread_cond_wait(condition, mutex));
+  thread = check_thread(L, 1);
+  if (!thread->joined) {
+    fprintf(stderr, "Lua/APR joining child thread from __gc() hook ..\n");
+    status = apr_thread_join(&unused, thread->handle);
+    if (status != APR_SUCCESS) {
+      char message[LUA_APR_MSGSIZE];
+      apr_strerror(status, message, count(message));
+      fprintf(stderr, "Lua/APR failed to join thread: %s\n", message);
+    } else if (thread->status == TS_ERROR) {
+      fprintf(stderr, "Lua/APR thread exited with error: %s\n", (char*)thread->output);
     }
-    LUA_APR_THREAD_MSG("%s", "Looks like all child threads finished?!");
-    threads_unlock(NULL, APR_SUCCESS);
   }
+  thread_destroy(thread);
+
+  return 0;
 }
 
 /* Lua/APR thread object metadata {{{1 */
 
 static luaL_Reg thread_methods[] = {
-  { "detach", thread_detach },
   { "join", thread_join },
   { "status", thread_status },
   { NULL, NULL }
