@@ -3,7 +3,7 @@
  Test infrastructure for the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: March 27, 2011
+ Last Change: July 2, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -34,6 +34,13 @@ function helpers.warning(s, ...) -- {{{1
   io.stderr:flush()
 end
 
+function helpers.try(body, errorhandler) -- {{{1
+  local status, value = pcall(body)
+  if status then return true end
+  errorhandler(value)
+  return false
+end
+
 function helpers.filedefined() -- {{{1
   local info = assert(debug.getinfo(2, 'S'))
   return info.source:sub(2)
@@ -81,14 +88,23 @@ end
 local tmpnum = 1
 local tmpdir = assert(apr.temp_dir_get())
 
+local function tmpname(tmpnum)
+  return apr.filepath_merge(tmpdir, 'lua-apr-tempfile-' .. tmpnum)
+end
+
 function helpers.tmpname() -- {{{1
-  local name = 'lua-apr-tempfile-' .. tmpnum
-  local file = apr.filepath_merge(tmpdir, name)
+  local file = tmpname(tmpnum)
   apr.file_remove(file)
   tmpnum = tmpnum + 1
   return file
 end
 
+function helpers.cleanup() -- {{{1
+  for i = 1, tmpnum do
+    apr.file_remove(tmpname(i))
+  end
+end
+
 function helpers.readfile(path) -- {{{1
   local handle = assert(io.open(path, 'r'))
   local data = assert(handle:read '*all')
@@ -109,6 +125,19 @@ function helpers.writable(directory) -- {{{1
   return status
 end
 
+local escapes = { ['\r'] = '\\r', ['\n'] = '\\n', ['"'] = '\\"', ['\0'] = '\\0' }
+
+function helpers.formatvalue(v) -- {{{1
+  if type(v) == 'number' then
+    local s = string.format('%.99f', v)
+    return s:find '%.' and (s:gsub('0+$', '0')) or s
+  elseif type(v) == 'string' then
+    return '"' .. v:gsub('[\r\n"%z]', escapes) .. '"'
+  else
+    return tostring(v)
+  end
+end
+
 -- }}}1
 
 return helpers
